@@ -35,20 +35,13 @@ namespace Beagle.Filters {
 		public FilterText ()
 		{
 			AddSupportedMimeType ("text/plain");
+			SnippetMode = true;
 		}
 
-		StreamReader reader;
-
-		override protected void DoOpen (FileInfo info)
-		{
-			reader = info.OpenText ();
-		}
-		
 		override protected void DoPull ()
 		{
-			string str = reader.ReadLine ();
+			string str = TextReader.ReadLine ();
 			if (str == null) {
-				reader.Close ();
 				Finished ();
 			} else {
 				AppendText (str);
