@@ -38,19 +38,32 @@ namespace Beagle.Daemon {
 
 		private Scheduler scheduler = Scheduler.Global;
 
+		private string index_dir;
 		private LuceneDriver driver;
 		private LuceneTaskCollector collector;
+		private IFileAttributesStore fa_store;
 
 		public LuceneQueryable (string index_name)
 		{
-			string index_dir = Path.Combine (PathFinder.RootDir, index_name);
+			index_dir = Path.Combine (PathFinder.RootDir, index_name);
 
 			driver = new LuceneDriver (index_dir);
-			driver.ChangedEvent += OnDriverChanged; 
+			fa_store = BuildFileAttributesStore (driver.Fingerprint);
+			driver.FileAttributesStore = fa_store;
+			driver.ChangedEvent += OnDriverChanged;
 
 			collector = new LuceneTaskCollector (driver);
 		}
 
+		virtual protected IFileAttributesStore BuildFileAttributesStore (string index_fingerprint)
+		{
+			return new FileAttributesStore_ExtendedAttribute (index_fingerprint);
+		}
+
+		protected string IndexDirectory {
+			get { return index_dir; }
+		}
+
 		protected LuceneDriver Driver {
 			get { return driver; }
 		}
@@ -59,6 +72,10 @@ namespace Beagle.Daemon {
 			get { return scheduler; }
 		}
 
+		public IFileAttributesStore FileAttributesStore {
+			get { return fa_store; }
+		}
+
 		/////////////////////////////////////////
 
 		private class ChangeData : IQueryableChangeData {
@@ -134,6 +151,11 @@ namespace Beagle.Daemon {
 			return "implement me!";
 		}
 
+		public virtual int GetItemCount ()
+		{
+			return Driver.GetItemCount ();
+		}
+
 		//////////////////////////////////////////////////////////////////////////////////
 
 		//
@@ -148,14 +170,20 @@ namespace Beagle.Daemon {
 			return task;
 		}
 
-		public Scheduler.Task NewAddTask (IIndexableGenerator generator)
+		public Scheduler.Task NewAddTask (IIndexableGenerator generator, Scheduler.Hook generator_hook)
 		{
 			LuceneTask task;
 			task = new LuceneTask (Driver, generator);
 			task.Priority = Scheduler.Priority.Generator;
+			task.GeneratorHook = generator_hook;
 			return task;
 		}
 
+		public Scheduler.Task NewAddTask (IIndexableGenerator generator)
+		{
+			return this.NewAddTask (generator, null);
+		}
+
 		public Scheduler.Task NewRemoveTask (Uri uri)
 		{
 			LuceneTask task;
@@ -164,6 +192,13 @@ namespace Beagle.Daemon {
 			return task;
 		}
 
+		public Scheduler.Task NewTaskFromHook (Scheduler.TaskHook hook)
+		{
+			Scheduler.Task task = Scheduler.TaskFromHook (hook);
+			task.Collector = collector;
+			return task;
+		}
+
 		//////////////////////////////////////////////////////////////////////////////////
 
 		private class LuceneTaskCollector : Scheduler.ITaskCollector {
@@ -220,6 +255,10 @@ namespace Beagle.Daemon {
 			// hard-wired
 			const int hard_wired_generation_count = 30;
 
+			// Hook to be invoked after the IIndexableGenerator
+			// has finished processing a batch of Indexables,
+			// just prior to flushing the driver.
+			public Scheduler.Hook GeneratorHook;
 
 			public LuceneTask (LuceneDriver driver, Indexable indexable) // Add
 			{
@@ -261,7 +300,8 @@ namespace Beagle.Daemon {
 					// get re-scheduled after it is run.
 					Reschedule = true;
 
-					for (int count = 0; count < hard_wired_generation_count; ++count) {
+					int count;
+					for (count = 0; count < hard_wired_generation_count; ++count) {
 						if (!generator.HasNextIndexable ()) {
 							// ...except if there is no more work to do, of course.
 							Reschedule = false;
@@ -278,7 +318,12 @@ namespace Beagle.Daemon {
 							driver.Add (generated);
 					}
 
+					if (count > 0 && this.GeneratorHook != null)
+						this.GeneratorHook ();
+
 					driver.Flush ();
+					if (driver.NeedsOptimize)
+						driver.Optimize ();
 				}
 			}
 		}
